#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#------------------------------------------------------------------------------------------------
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
# IT WAS CREATED AND TESTED IN A WINDOWS ENVIRONMENT ONLY AND MAY NOT FUNCTION ON A MAC.
#------------------------------------------------------------------------------------------------
# THIS PLUGIN IS ANOTHER FIGMENT OF MY IMAGENATION AND IS NOT BASED ON OR COMPLY WITH ANYTHING! 
#------------------------------------------------------------------------------------------------
#    Name:	Rotated Circle_Cylinder
#      By:	sdmitch
#   Usage:	Create a circle or cylinder in a defined direction and radius.
#		 Note:	The default is to create groups, use the Ctrl key to toggle group creation.  
#						Double click on face to define direction.  MAC uses Alt key to toggle grouping.
#    Date:	Jun 2012
#------------------------------------------------------------------------------------------------
#
require 'Sketchup'



class ThreePointCylinder

	def initialize
		@mod=Sketchup.active_model
		@ent=@mod.entities
		@sel=@mod.selection
		@guide_line = nil
		@num_sides = 24 if !@num_sides
		@ip=Sketchup::InputPoint.new
		@create="Circle" if !@create
		@group_coc = true
		case RUBY_PLATFORM
			when /mswin/ : @toggle = '(Ctrl)'
			when /darwin/ : @toggle = '(Alt)'
		end
		self.dialog
	end
	
	def dialog
		inp=inputbox(["Create:"],[@create],["Circle|Circle+Face|Cylinder|Cylinder+Guide"],"Circle or Cylinder")
		if inp
			@create=inp[0];self.reset
		else
			onCancel(nil,nil)
		end
	end
	
	def reset
		@status = 0
		@status_text = "Pick Center of #{@create}, Keyin sides, Right Click to change #{@create}, #{@toggle}=grouping:#{@group_coc ? 'Off' : 'On'}, Esc to exit."
		@vcb_label = "#{@create} Sides:"
		@vcb_value = @num_sides.to_s
		if(@guide_line!=nil)
			Sketchup.active_model.abort_operation # in case operation aborted.
			@guide_line=nil
		end
	end
	
	def onMouseMove(flags, x, y, view)
		@ip.pick view,x,y; view.tooltip = @ip.tooltip; view.refresh
		Sketchup::set_status_text @status_text
		Sketchup::set_status_text(@vcb_label,SB_VCB_LABEL)
		Sketchup::set_status_text(@vcb_value,SB_VCB_VALUE)
	end
	
	def onLButtonDown(flags, x, y, view)
		if @status==0
			@center = @ip.position;@tpts=[];@normal=nil
			@status_text = "Define direction/length of the #{@create}, Right click in space to abort, #{@toggle}=grouping:#{@group_coc ? 'Off' : 'On'}, Esc to exit."
			@vcb_label = "Length:";@vcb_value = "";@status = 1
			@mod.start_operation "Rotated #{@create}"
		elsif @status==1
			if @normal
				@end_pt = @ip.position
				@normal.length = @center.distance @end_pt
			else
				@end_pt = @ip.position
				@normal = @center.vector_to(@end_pt)
			end
			@guide_line.erase! if @guide_line
			@guide_line = @ent.add_cline(@end_pt,@normal.axes[1])
			@status_text = "Define or Keyin the Radius of the #{@create}, Right click in space to abort, #{@toggle}=grouping:#{@group_coc ? 'Off' : 'On'}, Esc to exit."
			@vcb_label = "#{@create} Radius:";@vcb_value = "";@status = 2
		elsif @status==2
			@guide_line.erase!
			self.create_cylinder
			@mod.commit_operation
			self.reset
		end
	end

	def onLButtonDoubleClick(flags, x, y, view)
		ph=view.pick_helper; ph.do_pick x,y; picks=ph.all_picked
		if @status==1
			if @ip.face
				@normal=@ip.face.normal; @corg = nil
				picks.each{|p| @corg=p if p.class==Sketchup::ComponentInstance||p.class==Sketchup::Group}
				@normal.transform!(@corg.transformation) if @corg
				@guide_line=@ent.add_cline(@center,@normal)
				@status=2 if @create=="Circle" || @create=="Circle+Face"
			end
		end
	end
	
	def onRButtonDown(flags, x, y, view)
		case @status
			when 0 : self.dialog
			when 1..2 : self.reset
		end
	end

	def create_cylinder
		@group_coc ? (grp=@ent.add_group; gent=grp.entities) : gent = @ent
		if @create=="Cylinder" || @create=="Cylinder+Guide"
			if @group_coc
				hgt=@center.distance(@end_pt); vec=@center.vector_to(@end_pt)
				cir=gent.add_circle([0,0,hgt],Z_AXIS,@rad,@num_sides)
				cir[0].find_faces; face=cir[0].faces[0];	face.pushpull -hgt
				gent.add_cline([0,0,0],[0,0,hgt]) if @create=="Cylinder+Guide"
				v1,v2,v3=vec.axes;v1.reverse! if(v1.cross(v2).dot(v3)<0)
				tr=Geom::Transformation.new(v1,v2,v3,@center)
				grp.transform!(tr);#orient the group axis to match the group side directions
			else
				cir=gent.add_circle(@center,@normal,@rad,@num_sides)
				cir[0].find_faces;face=cir[0].faces[0]
				height = @center.distance(@end_pt)
				height = -height if @normal.angle_between(face.normal) > Math::PI/2
				face.pushpull height
				gent.add_cline(@center,@end_pt) if @create=="Cylinder+Guide"
			end
		else
			if @group_coc
				cir=gent.add_circle([0,0,0],Z_AXIS,@rad,@num_sides)
				cir[0].find_faces if @create=="Circle+Face"
				vec=@center.vector_to(@end_pt);v1,v2,v3=vec.axes;v1.reverse! if(v1.cross(v2).dot(v3)<0)
				tr=Geom::Transformation.new(v1,v2,v3,@center)
				grp.transform!(tr);#orient the group axis to match the group side directions
			else
				cir=gent.add_circle(@center,@normal,@rad,@num_sides)
				cir[0].find_faces if @create=="Circle+Face"
			end
		end
	end

	def onUserText(text, view)
		begin
			@status==0 ? value = text.to_i : value = text.to_l
		rescue
			UI.beep;value=nil
		end
		return if !value
		case @status
			when 0
				@num_sides = value
				@vcb_label = "#{@create} Sides:"
				@vcb_value = @num_sides.to_s
			when 1
				@normal = @center.vector_to(@ip.position) if !@normal
				@normal.length = value;@end_pt = @center.offset(@normal)
				@status_text = "Define or Keyin the Radius of the #{@create}, Right click in space to abort, #{@toggle}=grouping:#{@group_coc ? 'Off' : 'On'}, Esc to exit."
				@vcb_label = "#{@create} Radius:";@vcb_value = "";@status = 2
			when 2
				@rad = value
				@mod.start_operation "Rotated #{@create}"
				self.create_cylinder
				@mod.commit_operation
				self.reset
		end
	end
	
	def onCancel(flags,view)
		Sketchup.send_action "selectSelectionTool:"
		view.invalidate if view
	end
	
	 def onKeyDown(key, repeat, flags, view)   
		 # puts "onKeyDown: key = " + key.to_s
		 if  key == VK_CONTROL || key == VK_ALT# Ctrl on PC & Alt on MAC
			@group_coc = !@group_coc
			case @status
				when 0 : @status_text = "Pick Center of #{@create}, Keyin sides, Right Click to change #{@create}, #{@toggle}=grouping:#{@group_coc ? 'Off' : 'On'}, Esc to exit."
				when 1 : @status_text = "Define direction/length, Right click in space to abort, #{@toggle}=grouping:#{@group_coc ? 'Off' : 'On'}, Esc to exit."
				when 2 : @status_text = "Define or Keyin Radius of #{@create}, Right click in space to abort, #{@toggle}=grouping:#{@group_coc ? 'Off' : 'On'}, Esc to exit."
			end
			Sketchup::set_status_text @status_text
		end
	 end	
	 
	def draw(view)
		if( @ip.valid? && @ip.display? )
			@ip.draw(view)
		end
		case @status
			when 1
				if @normal
					end_pt = @ip.position
					dist = @center.distance end_pt
				else
					end_pt = @ip.position
					dist = @center.distance end_pt
				end
				@vcb_value = "#{dist.to_l}"
				view.line_width=2
				view.line_stipple="-"
				view.set_color_from_line @center,@ip.position
				view.draw_line @center,end_pt
			when 2
				@rad = @ip.position.distance_to_line [@center,@normal]
				if @rad > 0.0
					@vcb_value = "#{@rad.to_l}"
					view.drawing_color="magenta"
					@bpts = self.points_on_circle(@center,@normal,@rad,@num_sides)
					view.draw GL_LINE_LOOP,@bpts
					if @create=="Cylinder" || @create=="Cylinder+Guide"
						for i in 0...@bpts.length
							@tpts[i]=@bpts[i].offset(@normal)
							view.draw_line @bpts[i],@tpts[i]
						end
						view.draw GL_LINE_LOOP,@tpts
					end
				end
		end
	end
	
	def  points_on_circle(center, normal, radius, numseg)
	# --------------------------------------------------------------------------------
	#	Function for generating points of a circle
	# --------------------------------------------------------------------------------
		xaxis,yaxis,zaxis = normal.axes# Get the x and y axes
		xaxis.length = radius;yaxis.length = radius
		da = (Math::PI * 2) / numseg;pts = []
		for i in (0...numseg)
			angle = i * da;cosa = Math.cos(angle);sina = Math.sin(angle)
			vec = Geom::Vector3d.linear_combination(cosa, xaxis, sina, yaxis)
			pts.push(center + vec)
		end
		return pts
	end
	
end
